package com.kdanmobile.reader

import android.annotation.SuppressLint
import com.kdanmobile.kmpdfkit.manager.KMPDFFactory
import com.kdanmobile.kmpdfkit.manager.controller.KMPDFDocumentController

class ReaderModel {
    private var filename: String? = null
    var password: String = ""
        private set

    var kmpdfFactory: KMPDFFactory? = null
        private set

    @SuppressLint("StaticFieldLeak")
    var kmpdfDocumentController: KMPDFDocumentController? = null
        private set

    var onPdfChangedListener: OnPdfChangedListener? = null

    var referenceCount = 0

    @Synchronized
    fun initKMPDFFactory(kmpdfFactory: KMPDFFactory?, filename: String?, password: String = "") {

        this.kmpdfFactory?.run {
            destroy()
        }

        this.kmpdfFactory = kmpdfFactory
        this.filename = filename
        this.password = password
    }

    fun initKMPDFDocumentController() {
        kmpdfDocumentController = kmpdfFactory?.getController(KMPDFFactory.ControllerType.DOCUMENT) as KMPDFDocumentController
    }

    @Synchronized
    fun destroy() {
        filename = null
        password = ""
        kmpdfFactory?.destroy()
        kmpdfFactory = null
        kmpdfDocumentController = null
    }

    fun register() {
        referenceCount++
    }

    fun unregister() {
        referenceCount--
        if (referenceCount == 0) {
            destroy()
        }
    }
}